module Easy
  class Resource < ActiveResource::Base

    def self.configure(conf)
      case conf
      when ::Hash
        if conf[:url].present? && conf[:token].present?
          configure_bearer(conf[:url], conf[:token])
        elsif conf[:url].present?
          self.site = conf[:url]
        end
      when ::OAuth2::AccessToken
        configure_bearer(conf.client.site, conf.token)
      else
        # do nothing, unknown configuration
      end
    end

    def self.configure_bearer(url, token)
      self.site = url
      connection.auth_type = :bearer
      connection.bearer_token = token
    end

    def becomes(entity_class)
      model = entity_class.new
      model.attributes = attributes
      model
    end

    def patch(method_name, options = {}, body = "")
      if method_name.nil?
        connection.patch(element_path(options), body, self.class.headers)
      else
        super
      end
    end

    def update_column(attribute, value)
      update_columns(attribute.to_sym => value)
    end

    def update_columns(attributes)
      patch(nil, {}, encode_attributes(attributes))
    end

    protected

    def encode_attributes(attributes)
      case self.class.format.extension
      when "json"
        attributes.to_json(include_root_in_json ? { root: self.class.element_name } : {})
      when "xml"
        attributes.to_xml(root: self.class.element_name)
      end
    end

  end
end
