module Easy
  module Resources
    module Redmine
      class EasyWebApplication < RedmineBase
        self.element_name = 'easy_web_application'

        self.schema = {
          url: :string,
          git_repository: :string,
          git_branch: :string,
          db_url: :string,
          files_url: :string,
          issue_id: :integer,
          easy_crm_case_id: :integer,
          easy_contact_id: :integer,
          aasm_event: :string,
          notes: :string,
          expire_at: :date,
          custom_repository: :boolean,
          custom_domain: :boolean,
        }

        belongs_to :easy_contact, class_name: 'Easy::Resources::Redmine::EasyContact'
        belongs_to :easy_crm_case, class_name: 'Easy::Resources::Redmine::EasyCrmCase'
        belongs_to :easy_server, class_name: 'Easy::Resources::Redmine::EasyServer'
        belongs_to :issue, class_name: 'Easy::Resources::Redmine::Issue'

        has_many :custom_fields, class_name: 'Easy::Resources::Redmine::CustomField'
        has_many :easy_web_application_activities, class_name: 'Easy::Resources::Redmine::EasyWebApplicationActivity'
        has_one :easy_web_application_activity, class_name: 'Easy::Resources::Redmine::EasyWebApplicationActivity'

        def name
          url
        end

        # @!group ESko specific custom_fields

        # @return [Boolean,nil]
        def application_moves_on_request?
          custom_fields.find { |cf| cf.internal_name == "application_move_on_request" }&.cast_value
        end

        # @return [Boolean,nil]
        def application_updates_on_request?
          custom_fields.find { |cf| cf.internal_name == "application_updates_on_request" }&.cast_value
        end

      end
    end
  end
end
