module Easy
  module Resources
    module Redmine
      class RedmineBase < ::Easy::Resource
        self.include_root_in_json = true
        self.format               = :xml
        add_response_method :http_response

        class_attribute :api_key

        class << self

          # @param [Hash, nil] conf
          def configure(conf = nil)
            conf ||= ::Easy.config.sites.redmine

            self.api_key = conf[:api_key]

            super(conf)
          end

          def headers
            h                      = super
            h['X-Redmine-API-Key'] = api_key
            h
          end

          # Parse value of TOTAL amount of entities from headers or response
          # @return [Integer]
          def retrieve_total(response)
            (response['Total'] || response.body[/total_count[:="]+(\d+)/] && Regexp.last_match(1)).to_i
          end

          def get_all(**params, &block)
            records      = results = all(params: params)
            response     = results.http_response
            total        = retrieve_total(response)
            current_page = 1

            while records.any?
              records.each(&block) if block_given?
              break if results.size >= total

              self.format = :xml

              records = all(params: params.merge(page: (current_page += 1)))
              results.concat(records)
            end

            results
          end

        end

        def errors
          @errors ||= Errors.new(self)
        end

      end
    end
  end
end
